#include "stdafx.h"
#include <errno.h>
#include <direct.h>
#include "unzip\unzip.h"
#include "extractor.h"

#define WRITEBUFFERSIZE (8192)

CExtractor::CExtractor(CHListBox *listbox)
{
	log = listbox;
}

CExtractor::~CExtractor()
{
}

bool CExtractor::ExtractZip(CString zip, CString where)
{
	CString filename_try;
	unzFile uf = NULL;

	filename_try = zip + ".zip";

	uf = unzOpen(zip);

    if (uf == NULL)
	{
		if ( (uf = unzOpen(filename_try)) == NULL)
		{
			log->AddString("Cannot open " + zip + " or " + zip + ".zip");
			return false;
		}
	}
    log->AddString(filename_try + " opened");

	bool retv = Extract(uf, where);
    unzCloseCurrentFile(uf);
	unzClose(uf);
    return retv;
}

void CExtractor::LogErrorNumber(int err, CString where)
{
	char buf[1024];
	StringCchPrintf(buf,1024,"Error %d with zipfile in ",err);
	log->AddString((CString)buf + where);
}

void CExtractor::ChangeFileDate(const char *filename, uLong dosdate, tm_unz tmu_date)
{
	HANDLE hFile;
	FILETIME ftm,ftLocal,ftCreate,ftLastAcc,ftLastWrite;

	hFile = CreateFile(filename,GENERIC_READ | GENERIC_WRITE,
						0,NULL,OPEN_EXISTING,0,NULL);
	GetFileTime(hFile,&ftCreate,&ftLastAcc,&ftLastWrite);
	DosDateTimeToFileTime((WORD)(dosdate>>16),(WORD)dosdate,&ftLocal);
	LocalFileTimeToFileTime(&ftLocal,&ftm);
	SetFileTime(hFile,&ftm,&ftLastAcc,&ftm);
	CloseHandle(hFile);
}


bool CExtractor::MakeDir (char *newdir)
{
	char *buffer;
	char *p;
	int  len = (int)strlen(newdir);

	if (len <= 0)
		return true;

	buffer = (char*)malloc(len+1);
	StringCchCopy(buffer, len+1, newdir);

	if (buffer[len-1] == '/')
	{
		buffer[len-1] = '\0';
	}
	if (CreateDirectory(buffer,NULL))
	{
		free(buffer);
		return true;
	}

	p = buffer+1;
	while (true)
	{
		char hold;

		while(*p && *p != '\\' && *p != '/') p++;
		hold = *p;
		*p = 0;
		if ((mkdir(buffer) == -1) && (errno == ENOENT))
		{
			log->AddString((CString)"couldn't create directory " + buffer);
			free(buffer);
			return false;
		}
		if (hold == 0) break;
		*p++ = hold;
	}
	free(buffer);
	return true;
}

bool CExtractor::ExtractCurrentFile(unzFile uf, CString where)
{
	char filename_inzip[256];
	char* filename_withoutpath;
    char* p;
    int err = UNZ_OK;
    FILE *fout=NULL;
    void* buf;
    uInt size_buf;
	char szCurDir[1024];

	GetCurrentDirectory(1024,szCurDir);
	SetCurrentDirectory(where);

    unz_file_info file_info;
    uLong ratio=0;
    err = unzGetCurrentFileInfo(uf,&file_info,filename_inzip,sizeof(filename_inzip),NULL,0,NULL,0);

    if (err != UNZ_OK)
    {
		LogErrorNumber(err, "unzGetCurrentFileInfo");
		SetCurrentDirectory(szCurDir);
        return false;
    }

    size_buf = WRITEBUFFERSIZE;
    buf = (void*)malloc(size_buf);
    if (buf==NULL)
    {
        log->AddString("Error allocating memory");
		SetCurrentDirectory(szCurDir);
        return false;
    }

    p = filename_withoutpath = filename_inzip;
    while ((*p) != '\0')
    {
        if (((*p)=='/') || ((*p)=='\\'))
            filename_withoutpath = p+1;
        p++;
    }

    if ((*filename_withoutpath) == '\0')
    {
		log->AddString((CString)"creating directory: " + filename_inzip);
		mkdir(filename_inzip);
		err = UNZ_OK;
    }
    else
    {
        const char* write_filename;
		write_filename = filename_inzip;

		CString newname = filename_inzip;
		if (newname.Left(1) == "\\")
		{
			newname = newname.Mid(2);
		}
		newname = where + newname;

        if (err == UNZ_OK)
        {
            fout=fopen(write_filename,"wb");

            // some zipfile don't contain directory alone before file
            if ((fout == NULL) && (filename_withoutpath != (char*)filename_inzip))
            {
                char c = *(filename_withoutpath-1);
                *(filename_withoutpath-1)='\0';
                MakeDir((char *)write_filename);
                *(filename_withoutpath-1)=c;
                fout=fopen(write_filename,"wb");
            }

            if (fout==NULL)
            {
                log->AddString((CString)"error opening " + write_filename);
            }
        }

        if (fout!=NULL)
        {
            log->AddString((CString)" extracting: " + write_filename);

            do
            {
                err = unzReadCurrentFile(uf,buf,size_buf);
                if (err<0)
                {				
                    LogErrorNumber(err,"unzReadCurrentFile");
                    break;
                }
                if (err>0)
				{
                    if (fwrite(buf,err,1,fout) != 1)
                    {
                        log->AddString("Error in writing extracted file");
                        err=UNZ_ERRNO;
                        break;
                    }
				}
            }
            while (err>0);
            if (fout)
                    fclose(fout);

            if (err==0)
                ChangeFileDate(write_filename,file_info.dosDate, file_info.tmu_date);
        }

        if (err==UNZ_OK)
        {
            err = unzCloseCurrentFile (uf);
            if (err!=UNZ_OK)
            {
				LogErrorNumber(err,"unzCloseCurrentFile");
            }
        }
        else
            unzCloseCurrentFile(uf);
    }

    free(buf);
	SetCurrentDirectory(szCurDir);
    return err == UNZ_OK;
}


bool CExtractor::Extract(unzFile uf, CString where)
{
    uLong i;
    unz_global_info gi;
    int err;
    FILE* fout=NULL;

    err = unzGetGlobalInfo (uf,&gi);
    if (err != UNZ_OK)
	{
		LogErrorNumber(err,"unzGetGlobalInfo");
		return false;
	}

    for (i=0;i<gi.number_entry;i++)
    {
		unzOpenCurrentFile(uf);
        if (!ExtractCurrentFile(uf, where)) return false;

        if ((i+1)<gi.number_entry)
        {
			unzCloseCurrentFile(uf);
            err = unzGoToNextFile(uf);
            if (err != UNZ_OK)
            {
				LogErrorNumber(err,"unzGoToNextFile");
                return false;
            }
        }
    }

    return true;
}
