// OptionEvarConfig.cpp : implementation file
//

#include "stdafx.h"
#include "base.h"
#include "OptionEvarConfig.h"
#include "utils.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const struct EVarInfo
{
	char name[20];	// variable name
	char desc[30];	// text description of a variable
	int  res_sid;	// resource id for evar caption
	int  res_eid;	// resource id for evar editbox
} evar_info[] = 
{
	{"FileManager",	"File manager",		IDC_FILEMANAGER,	IDC_EFILEMANAGER},
	{"TxtEditor",	"Text editor",		IDC_TEXTEDITOR,		IDC_ETEXTEDITOR},
	{"CmdPrompt",	"Command prompt",	IDC_CMDLINE,		IDC_ECMDLINE},
	{"AudioPlayer",	"Audio player",		IDC_AUDIOPLAYER,	IDC_EAUDIOPLAYER},
	{"MediaPlayer",	"Media player",		IDC_MEDIAPLAYER,	IDC_EMEDIAPLAYER},
	{"GfxViewer",	"Graphics viewer",	IDC_VIEWER,			IDC_EVIEWER},
	{"GfxEditor",	"Graphics editor",	IDC_GRAPHICSEDITOR,	IDC_EGRAPHICSEDITOR},
	{"Browser",		"Browser",			IDC_BROWSER,		IDC_EBROWSER},
	{"Email",		"E-mail client",	IDC_MAILCLIENT,		IDC_EMAILCLIENT},
	{"IRC",			"IRC client",		IDC_IRCCLIENT,		IDC_EIRCCLIENT},
	{"FTP",			"FTP client",		IDC_FTPCLIENT,		IDC_EFTPCLIENT},
	{"IM",			"IM client",		IDC_IMCLIENT,		IDC_EIMCLIENT}
};

/////////////////////////////////////////////////////////////////////////////
// COptionEvarConfig dialog


COptionEvarConfig::COptionEvarConfig(CWnd* pParent /*=NULL*/)
	: CDialog(COptionEvarConfig::IDD, pParent)
{
	//{{AFX_DATA_INIT(COptionEvarConfig)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void COptionEvarConfig::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COptionEvarConfig)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(COptionEvarConfig, CDialog)
	//{{AFX_MSG_MAP(COptionEvarConfig)
	ON_BN_CLICKED(IDC_BFILEMANAGER, OnButtonFileManager)
	ON_BN_CLICKED(IDC_BTEXTEDITOR, OnBTextEditor)
	ON_BN_CLICKED(IDC_BCMDLINE, OnButtonCommandLine)
	ON_BN_CLICKED(IDC_BAUDIOPLAYER, OnButtonAudioPlayer)
	ON_BN_CLICKED(IDC_BMEDIAPLAYER, OnButtonMediaPlayer)
	ON_BN_CLICKED(IDC_BVIEWER, OnButtonViewer)
	ON_BN_CLICKED(IDC_BGRAPHICSEDITOR, OnButtonGraphicsEditor)
	ON_BN_CLICKED(IDC_BBROWSER, OnButtonBrowser)
	ON_BN_CLICKED(IDC_BMAILCLIENT, OnButtonMailClient)
	ON_BN_CLICKED(IDC_BIRCCLIENT, OnButtonIRCClient)
	ON_BN_CLICKED(IDC_BFTPCLIENT, OnButtonFTPClient)
	ON_BN_CLICKED(IDC_BIMCLIENT, OnButtonIMClient)
	ON_BN_CLICKED(IDC_DISCARDCHANGES, OnDiscardChanges)
	ON_BN_CLICKED(IDC_SAVEEVARS, OnSaveEvars)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


void COptionEvarConfig::SelectVariable(int idb)
{
	for (int i=0; i<EVARS_NUM; i++)
	{
		if (idb == evar_info[i].res_eid)
		{
			CString filt =	"Executables (*.exe)|*.exe|";
			CFileDialog fileDlg(TRUE,"*.exe","*.exe",NULL,filt,this);

			if (utils.bIsNT50)
			{
				fileDlg.m_ofn.lStructSize = sizeof(OPENFILENAMEEX);
			}
			fileDlg.m_ofn.Flags |= OFN_FILEMUSTEXIST;
			CString s = evar_info[i].desc;
			s.MakeLower();
			fileDlg.m_ofn.lpstrTitle = "Select your " + s;
	
			if (fileDlg.DoModal() == IDOK) 
			{
				CWaitCursor bob;
				SetDlgItemText(evar_info[i].res_eid,fileDlg.GetPathName());
			}
			return;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// COptionEvarConfig message handlers

BOOL COptionEvarConfig::OnInitDialog() 
{
	CDialog::OnInitDialog();

	CString evarsrc = utils.personalDir + "evars.rc";
	CString value;

	for (int i=0; i<EVARS_NUM; i++)
	{
		value = utils.ReadOptionFromRC(evarsrc,evar_info[i].name);
		if (value == "")
		{
			value = "...";
		}
		lastvalue[i] = value;
		SetDlgItemText(evar_info[i].res_eid,value);
	}
	
	return TRUE;
}

void COptionEvarConfig::OnDiscardChanges() 
{
	for (int i=0; i<EVARS_NUM; i++)
	{
		SetDlgItemText(evar_info[i].res_eid,lastvalue[i]);
	}	
}

void COptionEvarConfig::OnSaveEvars() 
{
	CString evarsrc = utils.personalDir + "evars.rc";
	CString value;
	for (int i=0; i<EVARS_NUM; i++)
	{
		//SetDlgItemText(evar_info[i].res_eid,lastvalue[i]);
		GetDlgItemText(evar_info[i].res_eid,value);
		if (value == "")
		{
			value = "...";
		}
		utils.Trim(value);
		if (value.Left(1) != '\"')
		{
			value = "\"" + value + "\"";
		}
		if (!utils.WriteOptionToRC(evarsrc,evar_info[i].name,value))
		{
			AfxMessageBox("Cannot save changes in " + evarsrc);
			return;
		}
		lastvalue[i] = value;
	}
	AfxMessageBox("Evars saved successfully");
}

void COptionEvarConfig::OnButtonFileManager() 
{
	SelectVariable(IDC_EFILEMANAGER);
}

void COptionEvarConfig::OnBTextEditor() 
{
	SelectVariable(IDC_ETEXTEDITOR);
}

void COptionEvarConfig::OnButtonCommandLine() 
{
	SelectVariable(IDC_ECMDLINE);	
}

void COptionEvarConfig::OnButtonAudioPlayer() 
{
	SelectVariable(IDC_EAUDIOPLAYER);	
}

void COptionEvarConfig::OnButtonMediaPlayer() 
{
	SelectVariable(IDC_EMEDIAPLAYER);	
}

void COptionEvarConfig::OnButtonViewer() 
{
	SelectVariable(IDC_EVIEWER);		
}

void COptionEvarConfig::OnButtonGraphicsEditor() 
{
	SelectVariable(IDC_EGRAPHICSEDITOR);	
}

void COptionEvarConfig::OnButtonBrowser() 
{
	SelectVariable(IDC_EBROWSER);	
}

void COptionEvarConfig::OnButtonMailClient() 
{
	SelectVariable(IDC_EMAILCLIENT);	
}

void COptionEvarConfig::OnButtonIRCClient() 
{
	SelectVariable(IDC_EIRCCLIENT);	
}

void COptionEvarConfig::OnButtonFTPClient() 
{
	SelectVariable(IDC_EFTPCLIENT);	
}

void COptionEvarConfig::OnButtonIMClient() 
{
	SelectVariable(IDC_EIMCLIENT);	
}
