// ThemeInstaller.cpp: implementation of the CThemeInstaller class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "base.h"
#include "ThemeInstaller.h"
#include "OptionInstaller.h"
#include "utils.h"
#include "enbitmap.h"
#include "png_support.h"
#include "extractor.h"
#include "regkey.h"
#include <list>
#include <vector>
#include <algorithm>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

extern CbaseApp theApp;

#define REGKEY_THEMEINSTALLER	"SOFTWARE\\LiteStep\\Theme Installer"

CThemeInstaller themeinstaller;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CThemeInstaller::CThemeInstaller()
{
	ReadRegistry();
}

CThemeInstaller::~CThemeInstaller()
{

}

//////////////////////////////////////////////////////////////////////////////////
//
// Read Theme Installer's settings from the registry
//
bool CThemeInstaller::ReadRegistry()
{
	bEditThemevars		= ReadBoolFromRegistry("EditThemevars",true);
	bRevertPrompt		= ReadBoolFromRegistry("RevertPrompt",true);
	bOneWallpaper		= ReadBoolFromRegistry("OneWallpaper",true);
	bInstallWallpaper	= ReadBoolFromRegistry("InstallWallpaper",true);
	bInstallFont		= ReadBoolFromRegistry("InstallFonts",false);
	bWritePatch			= ReadBoolFromRegistry("WritePatch",false);
	bInstallWinamp2		= ReadBoolFromRegistry("InstallWinamp2",true);
	bInstallWinamp3		= ReadBoolFromRegistry("InstallWinamp3",true);
	bInstallWindowBlinds= ReadBoolFromRegistry("InstallWindowBlinds",true);
	szDefaultTheme	= ReadStringFromRegistry("DefaultTheme","austerity");
	return true;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Write Theme Installer's settings to the registry
//
bool CThemeInstaller::WriteRegistry()
{
	CRegKey key;
	if (key.Open(HKEY_CURRENT_USER,REGKEY_THEMEINSTALLER, KEY_WRITE) != ERROR_SUCCESS)
	{
		if (key.Create(HKEY_CURRENT_USER,REGKEY_THEMEINSTALLER) != ERROR_SUCCESS)
			return false;
	}
	key.SetValue(bEditThemevars			? "1" : "0", "EditThemevars");
	key.SetValue(bRevertPrompt			? "1" : "0", "RevertPrompt");
	key.SetValue(bOneWallpaper			? "1" : "0", "OneWallpaper");
	key.SetValue(bInstallWallpaper		? "1" : "0", "InstallWallpaper");
	key.SetValue(bInstallFont			? "1" : "0", "InstallFonts");
	key.SetValue(bInstallWinamp2		? "1" : "0", "InstallWinamp2");
	key.SetValue(bInstallWinamp3		? "1" : "0", "InstallWinamp3");
	key.SetValue(bInstallWindowBlinds	? "1" : "0", "InstallWindowBlinds");
	key.SetValue(bWritePatch			? "1" : "0", "WritePatch");
	key.SetValue(szDefaultTheme,"DefaultTheme");
	key.Close();
	return true;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Read a 'bool' setting from the registry
//   keyname is a name of 'HKCU\SOFTWARE\LiteStep\Theme Installer' subkey
//   defvalue is a default value that would be returned if key is unavailable
//
bool CThemeInstaller::ReadBoolFromRegistry(CString keyname, bool defvalue)
{
	CRegKey key;
	bool retv = defvalue;
	if (key.Open(HKEY_CURRENT_USER,REGKEY_THEMEINSTALLER,KEY_READ) == ERROR_SUCCESS)
	{
		char buf[1024];
		DWORD bufsize = 1024;
		if (key.QueryValue(buf,keyname,&bufsize) == ERROR_SUCCESS)
		{
			retv = (strcmp(buf,"1") == 0);
		}
		key.Close();
	}
	return retv;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Read a 'string' setting from the registry
//   keyname is a name of 'HKCU\SOFTWARE\LiteStep\Theme Installer' subkey
//   defvalue is a default value that would be returned if key is unavailable
//
CString CThemeInstaller::ReadStringFromRegistry(CString keyname, CString defvalue)
{
	CRegKey key;
	CString retv = defvalue;
	if (key.Open(HKEY_CURRENT_USER,REGKEY_THEMEINSTALLER,KEY_READ) == ERROR_SUCCESS)
	{
		char buf[1024];
		DWORD bufsize = 1024;
		if (key.QueryValue(buf,keyname,&bufsize) == ERROR_SUCCESS)
		{
			retv = buf;
		}
		key.Close();
	}
	return retv;
}


//////////////////////////////////////////////////////////////////////////////////
//
// The function parses theme.ini and then install theme wallpaper
//
bool CThemeInstaller::InstallWallpaper(CString themename)
{
	// this feature doesn't work for OTS1 themes... yet
	if (utils.IsThemeOTS1Compliant(themename))
	{
		if (utils.IsFileExist(utils.themesDir + "\\" + themename + "\\" + themename + ".thm"))
		{
			AfxMessageBox("Wallpaper installing skipped since I don\'t know how to parse XML files");
		}
		return false;
	}
	// read wallpaper file name from theme.ini
	CString wallfile = utils.ReadOptionFromINI(themename,"Wallpaper");
	if (wallfile == "") return true;
	CString wallstyle = utils.ReadOptionFromINI(themename,"WallpaperStyle");
	wallfile.TrimLeft('\"');
	wallfile.TrimRight('\"');
	wallstyle.TrimLeft('\"');
	wallstyle.TrimRight('\"');
	if (wallstyle == "") wallstyle = "stretched";
	// full filename
	wallfile = utils.themesDir + themename + "\\" + wallfile;
	if (bOneWallpaper)
		utils.InstallWallpaper(wallfile,wallstyle);
	{
		// save as .BMP into the \themes dir
		CString bmpwall = utils.themesDir + "wallpaper.bmp";
		// encode file
		CEnBitmap bm;
		HBITMAP hbm = NULL;
		bm.LoadImage(wallfile);
		if (bm.m_hObject != NULL)
			utils.SaveBitmapToFile((HBITMAP)bm,bmpwall);
		else
		{
			hbm = LoadFromPNG(wallfile);
			if (hbm == NULL)
				AfxMessageBox("Cannot load wallpaper. File is missed or corrupt");
			else
			{
				utils.SaveBitmapToFile(hbm,bmpwall);
				if (hbm != NULL) DeleteObject(hbm);
			}
		}
		utils.InstallWallpaper(bmpwall,wallstyle);
	}
	return true;
}

//////////////////////////////////////////////////////////////////////////
//
// Install _compatibility_patch.rc
//
bool CThemeInstaller::CompatibilityPatch(CString pathtotheme)
{
	CreateDirectory(pathtotheme + "config",NULL);
	CStdioFile f;
	if (!f.Open(pathtotheme + "config\\_compatibility_patch.rc",CFile::modeWrite | CFile::modeCreate))
		return false;
	f.WriteString(";------------------------------------------------------------------------------");
	f.WriteString(";   PLEASE HELP LITESTEP TO MOVE FORWARD, DON'T KEEP IT MIRED IN THE PAST. :(");
	f.WriteString(";");
	f.WriteString(";           PLEASE USE AND DEVELOP OTS2 COMPLIANT LITESTEP THEMES!!!");
	f.WriteString(";");
	f.WriteString(";                             http://o-t-s.sf.net                              ");
	f.WriteString(";------------------------------------------------------------------------------");
	f.WriteString("\nThemeIsOTS1Compliant    true");
	f.Close();
	return true;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Patch a theme file in accroding to the AngelKnight's algorithm
//
bool CThemeInstaller::PatchFileOTSv1(CString csFile)
{
	CStdioFile fin, fout, fmods;
	CString line, tmpline, modline;

	if (!fin.Open(csFile, CFile::modeRead))
	{
		return false;
	}
	if (!fout.Open(csFile+".tmp", CFile::modeWrite | CFile::modeCreate))
	{
		fin.Close();
		return false;
	}

	while (fin.ReadString(line))
	{
		tmpline = line;
		utils.Trim(tmpline);
		tmpline.MakeLower();
		if (tmpline.Left(10) == "loadmodule"  ||
			tmpline.Left(11) == "*loadmodule" ||
			tmpline.Left(7)  == "*module"     ||
			tmpline.Left(12) == "*wharfmodule")
		{
			if (tmpline.Find("jkey.dll") >= 0 || tmpline.Find("hotkey2.dll") >= 0) continue;
			fmods.Open(utils.toolDir + "lscpfiles\\update\\modcom.ini", CFile::modeRead);
			while (fmods.ReadString(modline))
			{
				int i = modline.Find('/');
				CString modName = modline.Left(i);
				CString modNLM  = modline.Mid(i + 1);
				modName.MakeLower();
				if (tmpline.Find(modName)>=0)
				{
					if (tmpline.Left(10) == "loadmodule")
						line = "*NetLoadModule " + modNLM;
					else if (tmpline.Left(11) == "*loadmodule")
						line = "*LoadModule \"$ModulesDir$" + modNLM + ".dll\"";
					else if (tmpline.Left(7) == "*module")
						line = "*Module \"$ModulesDir$" + modNLM + ".dll\"";
					else	//*WharfModule
					{
						if (line.Find(';') >= 0)	// strip comments
							line = line.Left(line.Find(';'));
						utils.TrimRight(line);
						int space = line.ReverseFind(' ');	// find last token
						int pret = line.ReverseFind('\t');
						if (space<0 || (pret>=0 && space<pret)) space = pret;
						line = line.Left(space + 1);	// remove the token
						line += "\"$ModulesDir$" + modNLM + ".dll\"";
					}
					break;
				}
			}
			fmods.Close();
		} else	if (tmpline.Find("personaldir") >= 0)
			line = "; " + line;
		else	if (tmpline.Mid(0,8) == "themedir")
			line = "; " + line;
		else	if (tmpline.Mid(0,10) == "modulesdir")
			line = "; " + line;
		else	if (tmpline.Find("personal.rc") >= 0)
			line = "; " + line;
		fout.WriteString(line + "\n");
	}
	fout.Close();
	fin.Close();
	fmods.Close();
	remove(csFile);
	rename(csFile + ".tmp",csFile);
	return true;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Enumerate all config files in the directory and it's subdirectories for patching
//
bool CThemeInstaller::PatchFolderOTSv1(CString csFolder)
{
	const int iExtNum = 2;
	const char *ext[] = {"*.rc","*.box"};

	HANDLE hFind;
	WIN32_FIND_DATA dataFind;
	BOOL bMoreFiles = TRUE;

	// directories
	hFind = FindFirstFile(csFolder + "*.*", &dataFind);
	while (hFind != INVALID_HANDLE_VALUE && bMoreFiles == TRUE)
	{
		if (dataFind.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		{
			if (strcmp(dataFind.cFileName, ".") && strcmp(dataFind.cFileName, ".."))
			{
				if (!PatchFolderOTSv1(csFolder + dataFind.cFileName + (CString)"\\"))
				{
					FindClose(hFind);
					return false;
				}
			}
		}
		bMoreFiles = FindNextFile(hFind, &dataFind);
	}
	FindClose(hFind);

	for (int i=0; i<iExtNum; i++)
	{
		bMoreFiles = TRUE;
		hFind = FindFirstFile(csFolder + ext[i], &dataFind);
		while (hFind != INVALID_HANDLE_VALUE && bMoreFiles == TRUE)
		{
			if (! (dataFind.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				if (!PatchFileOTSv1(csFolder + dataFind.cFileName))
				{
					FindClose(hFind);
					return false;
				}
			}
			bMoreFiles = FindNextFile(hFind, &dataFind);
		}
		FindClose(hFind);
	}
	return true;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Read wallpaper settings from OTSv1 xml schema
//
bool CThemeInstaller::ReadWallpaperSettings(CString csThmFile, CString &wall, CString &wallstyle)
{
	CStdioFile f;

	if (!f.Open(csThmFile,CFile::modeRead))
		return false;

	CString line;
	int pos;
	bool wfilefound = false, wstylefound = false;

	while (f.ReadString(line))
	{
		pos = line.Find("<image src=");
		if (pos >= 0)
		{
			line = line.Mid(pos + 11);
			line = line.Left(line.ReverseFind('/'));
			utils.Trim(line);
			line.TrimLeft('\"');
			line.TrimRight('\"');
			wall = line;
			wfilefound = true;
			continue;
		}
		pos = line.Find("<style>");
		if (pos >= 0)
		{
			line = line.Mid(pos + 7);
			line = line.Left(line.ReverseFind('/') - 1);
			utils.Trim(line);
			line.TrimLeft('\"');
			line.TrimRight('\"');
			if (line == "centre")
				wallstyle = "centered";
			else if (line == "tile")
				wallstyle = "tiled";
			else if (line == "stretch")
				wallstyle = "stretched";
			wstylefound = true;
		}
	}
	f.Close();
	return wfilefound && wstylefound;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Patch all theme files to try make them OTSv2 compatible
//
bool CThemeInstaller::PatchThemeOTSv1(CString pathtotheme)
{
	//bool rv = PatchFileOTSv1(pathtotheme + "step.rc");
	bool rv = PatchFolderOTSv1(pathtotheme);
	if (rv)
		CopyFile(pathtotheme + "step.rc", pathtotheme + "theme.rc", FALSE);
	utils.WriteOptionToRC(pathtotheme + "theme.rc", "OTSMajorVersion", "2");
	utils.WriteOptionToRC(pathtotheme + "theme.rc", "OTSMinorVersion", "0");

	CStdioFile thm, ini;
	CString csTheme = "";

	// find the first .thm-file. it must be not a directory! :)
	HANDLE hFind;
	WIN32_FIND_DATA dataFind;
	BOOL bMoreFiles = TRUE;
	hFind = FindFirstFile(pathtotheme + "*.thm", &dataFind);
	while (hFind != INVALID_HANDLE_VALUE && bMoreFiles == TRUE)
	{
		if (! (dataFind.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
		{
			csTheme = pathtotheme + dataFind.cFileName;	// found
			break;
		}
		bMoreFiles = FindNextFile(hFind, &dataFind);
	}
	FindClose(hFind);

	if (csTheme != "")
	{
		CString wall, wallstyle;
		if (ReadWallpaperSettings(csTheme,wall,wallstyle))
		{
			if (ini.Open(pathtotheme + "theme.ini", CFile::modeWrite | CFile::modeCreate))
			{
				ini.WriteString("[Install]\n");
				ini.WriteString("Wallpaper=\"" + wall + "\"\n");
				ini.WriteString("WallpaperStyle=\"" + wallstyle + "\"\n");
				ini.Close();
			}
		}
	}
	return rv;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Prepare OTSv1 theme for installing: write Omar's compatibility patch or patch the theme
//
bool CThemeInstaller::PrepareOTS1Theme(CString csThemeDir, CHListBox &log)
{
	log.AddString("Theme is OTS1 compliant");
	if (bWritePatch)
	{
		// installing Omar's LSInstaller compatibility patch
		if (!CompatibilityPatch(csThemeDir))
		{
			log.AddString("Cannot install compatibility patch to the theme");
			return false;
		}
		log.AddString("Compatibility patch installed successfully");
	} else
	{
		if (!PatchThemeOTSv1(csThemeDir))
		{
			log.AddString("Cannot patch a theme");
			return false;
		}
		log.AddString("Theme files successfully patched");
	}
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Compare OTSMajorVersion/OTSMinorVersion with OTSCfgMajorVersion/OTSCfgMinorVersion
//
bool CThemeInstaller::CheckOTSVersion(CString themerc, CHListBox &log)
{
	// checking the  OTS version
	CString themeotsmajor = utils.ReadOptionFromRC(themerc,"OTSMajorVersion");
	CString themeotsminor = utils.ReadOptionFromRC(themerc,"OTSMinorVersion");
	CString configotsmajor= utils.ReadOptionFromRC(utils.personalDir + "personal.rc","OTSCfgMajorVersion");
	CString configotsminor= utils.ReadOptionFromRC(utils.personalDir + "personal.rc","OTSCfgMinorVersion");
	if (themeotsmajor == "" || themeotsminor == "")
	{
		log.AddString("Theme is not OTS2 compliant!");
		log.AddString("OTSMajorVersion and OTSMinorVersion required but not specified");
		return false;
	}
	if (configotsmajor == "" || configotsminor == "")
	{
		log.AddString("Your personal .config files are not OTS2 compliant!");
		log.AddString("OTSCfgMajorVersion and OTSCfgMinorVersion required but not specified");
		return false;
	}
	if (themeotsmajor != configotsmajor || themeotsminor != configotsminor)
	{
		log.AddString("Theme required another version of config files!");
		log.AddString("Theme developed for OTS v" + themeotsmajor + "." + themeotsminor + " distro");
		log.AddString("But you have OTS v" + configotsmajor + "." + configotsminor + " personal files");
		return false;
	}
	return true;
}

/////////////////////////////////////////////////////////////////////
//
// install the theme font if nessesary
//
void CThemeInstaller::InstallFont(CString themename, CHListBox &log)
{
	CString fontfile = utils.ReadOptionFromINI(themename,"Font");
	if (fontfile != "")
	{
		CFontInstaller fontinstall;
		CString fontname = utils.ReadOptionFromINI(themename,"FontName");
		if (fontname == "")
		{
			log.AddString("Theme .ini is not written properly");
			log.AddString("Font file has been specified, but font name missed");
		}
		else
		{
			if (!fontinstall.InstallFont(utils.themesDir + themename + "\\" + fontfile, fontname))
			{
				log.AddString("Cannot install font named " + fontname);
				log.AddString(fontinstall.szErrorMessage);
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////
//
// Theme installer...
//  zip - theme archive file name
//  log - list that will be filled with 'debug' information
//  bDontSwitch - do not switch the new theme, just install it
//

bool CThemeInstaller::Install(CString zip, CHListBox &log, bool bDontSwitch)
{
	log.ResetContent();
	utils.Trim(zip);
	if (zip == "")
	{
		log.AddString("Theme archive not specified!");
		return false;
	}
	CString proto = zip.Left(7);
	proto.MakeLower();
	if (proto == "http://")
	{
		log.AddString("HTTP protocol currently not supported");
		return false;
	}
	CString ThemeName = zip;
	int left,right;
	int l1 = zip.ReverseFind('\\'), l2 =  zip.ReverseFind('/');
	left = (l1>l2) ? l1 : l2;
	right = zip.ReverseFind('.');
	if (left >= right || right<0)
	{
		log.AddString("Invalid archive name");
		return false;
	}
	if (left >= 0)
		ThemeName = zip.Mid(left+1,right-left-1);
	else
		ThemeName = zip.Left(right);
	log.AddString("Started installation of \'" + ThemeName + "\' theme...");

	CExtractor extractor(&log);

	char tempdir[MAX_PATH];
	GetTempPath(MAX_PATH,tempdir);
	CString tempfolder = tempdir;
	if (tempfolder.Right(1) != "\\") tempfolder += "\\";
	CreateDirectory(tempfolder + "lscp2",NULL);
	tempfolder += "lscp2\\" + ThemeName;
	utils.DeleteFolder(tempfolder);
	CreateDirectory(tempfolder,NULL);
	tempfolder += "\\";
	if (!extractor.ExtractZip(zip, tempfolder))
	{
		log.AddString("Cannot extract theme files :((");
		log.AddString("Probably you havn't enough space or system policy conflict occurs");
		return false;
	}
	log.AddString("Theme files unzipped successfully");

	// first let's try the first level files
	// some themers packed files in a wrong way
	CString themerc = tempfolder + "theme.rc";
	CString steprc = tempfolder + "step.rc";
	bool bStepRc = utils.IsFileExist(steprc);
	bool bThemeRc= utils.IsFileExist(themerc);
	if (!bStepRc && !bThemeRc)
	{
		// Ok, it was not an incorrect OTS theme archive
		// now we need to find the first dir in the temporary
		// folder since it is a main theme folder that contain
		// theme.rc, step.rc, etc...
		HANDLE hFind;
		WIN32_FIND_DATA dataFind;
		BOOL bMoreFiles = TRUE;
		hFind = FindFirstFile(tempfolder + "*.*", &dataFind);
		while (hFind != INVALID_HANDLE_VALUE && bMoreFiles == TRUE)
		{
			if (dataFind.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			{
				if (strcmp(dataFind.cFileName, ".") && strcmp(dataFind.cFileName, ".."))
				{
					// folder found
					ThemeName = dataFind.cFileName;
					tempfolder += ThemeName + "\\";
					themerc = tempfolder + "theme.rc";
					steprc  = tempfolder + "step.rc";
					bStepRc = utils.IsFileExist(steprc);
					bThemeRc= utils.IsFileExist(themerc);
					break;
				}
			}
			bMoreFiles = FindNextFile(hFind, &dataFind);
		}
		FindClose(hFind);
	}
	if (bThemeRc)	//OTS2 theme.rc found
	{
		if (!CheckOTSVersion(themerc,log))
		{
			utils.DeleteFolder(tempfolder.Left(tempfolder.GetLength()-1));
			return false;
		}
	}
	else
	{
		if (!bStepRc)
		{
			log.AddString("No step.rc or theme.rc found");
			log.AddString("Theme is not OTS comliant");
			utils.DeleteFolder(tempfolder.Left(tempfolder.GetLength()-1));
			return false;
		}
		themerc = steprc;
	}
	// Short information
	CString themename	= utils.ReadOptionFromRC(themerc,"ThemeName");
	CString themeauthor	= utils.ReadOptionFromRC(themerc,"ThemeAuthor");
	if (themename  != "") log.AddString("Theme name: "  + themename);
	if (themeauthor!= "") log.AddString("Theme author: "+ themeauthor);
	if (!bThemeRc) // is theme OTS1 compliant
	{
		if (!PrepareOTS1Theme(tempfolder,log))
		{
			utils.DeleteFolder(tempfolder.Left(tempfolder.GetLength()-1));
			return false;
		}
	}
	// move theme files from the temporary directory to the $ThemesDir$
	SHFILEOPSTRUCT shfop;
	char from[MAX_PATH+1];
	char to[MAX_PATH+1];
	StringCbCopy(from,MAX_PATH+1,tempfolder.Left(tempfolder.GetLength()-1));
	from[tempfolder.GetLength()] = 0;
	StringCbCopy(to,MAX_PATH+1,utils.themesDir.Left(utils.themesDir.GetLength()-1));
	to[utils.themesDir.GetLength()] = 0;
	shfop.hwnd = log.GetParent()->m_hWnd;
	shfop.wFunc = FO_MOVE;
	shfop.pFrom = from;
	shfop.pTo = to;
	shfop.fFlags = FOF_NOCONFIRMMKDIR;
	SHFileOperation(&shfop);

	if (bInstallFont)
	{
		InstallFont(ThemeName, log);
	}
	RefreshList();
	CString csThemeVars = utils.themesDir + ThemeName + "\\config\\themevars.rc";
	if (bEditThemevars && utils.IsFileExist(csThemeVars))
	{		
		SHELLEXECUTEINFO shinfo;
		shinfo.cbSize		=	sizeof(SHELLEXECUTEINFO);
		shinfo.fMask		=	SEE_MASK_NOCLOSEPROCESS | SEE_MASK_FLAG_DDEWAIT;
		shinfo.hwnd			=	NULL;
		shinfo.lpVerb		=	NULL;						// default action
		shinfo.lpFile		=	csThemeVars;				// themevars.rc
		shinfo.lpParameters	=	NULL;
		shinfo.lpDirectory	=	NULL;
		shinfo.nShow		=	SW_SHOWNORMAL;
		if (ShellExecuteEx(&shinfo))
		{
			if (shinfo.hInstApp >= (HINSTANCE)32)
			{
				theApp.m_pMainWnd->ShowWindow(SW_HIDE);
				WaitForSingleObject(shinfo.hProcess,INFINITE);
				theApp.m_pMainWnd->ShowWindow(SW_SHOW);
			}
		}
	}
	// switch to the successfully installed theme
	if (!bDontSwitch)
		Switch(ThemeName);
	else
		utils.ExecuteBang("!recycle");
	return true;
}


//////////////////////////////////////////////////////////////////////////////////
//
// Small theme switcher
//
bool CThemeInstaller::Switch(CString theme)
{
	CStdioFile f;
	CString csTheme = stricmp(theme,".default") ? theme : szDefaultTheme;

	CString themeselectrc = utils.themesDir + "themeselect.rc";
	if (!f.Open(themeselectrc ,CFile::modeWrite | CFile::modeCreate))
	{	
		AfxMessageBox("Can't open " + themeselectrc + " for writing");
		return false;
	}
	else
	{
		// write themeselect.rc
		f.WriteString(";------------------------------------------------------------------------------\n");
		f.WriteString(";   edit this evar to switch themes\n");
		f.WriteString(";------------------------------------------------------------------------------\n");
		f.WriteString("\nThemeDir \"$ThemesDir$" + csTheme + "\\\"\n");
		f.Close();
		// change wallpaper
		InstallWallpaper(csTheme);
		utils.ExecuteBang("!recycle");
		if (bRevertPrompt && stricmp(csTheme,szDefaultTheme) != 0)
		{
			if (IDYES == MessageBox(NULL,"Do you want to revert to the default theme (" + szDefaultTheme + ")?","LiteStep Control Panel 2 v1.0",MB_YESNO | MB_ICONQUESTION | MB_TOPMOST))
			{
				Switch(szDefaultTheme);
			}
		}
		return true;
	}
}


//////////////////////////////////////////////////////////////////////////////////
//
// Process toolDir variable and replace path to the LiteStep to $LiteStep$
//
void CThemeInstaller::PrepareToolDir()
{
	toolDir = utils.toolDir;
	if (strnicmp(toolDir, utils.litestepDir, utils.litestepDir.GetLength()) == 0)
	{
		toolDir = (CString)"$LiteStepDir$" + toolDir.Mid(utils.litestepDir.GetLength());
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
// Small theme switcher
//

// Return whether first element is greater than the second
bool CompareStrings(CString s1, CString s2)
{
   return stricmp(s1,s2) <= 0;
}

bool CThemeInstaller::RefreshList()
{
	CStdioFile f;
	if (!f.Open(utils.themesDir + "themeslist.rc",CFile::modeWrite | CFile::modeCreate))
	{
		AfxMessageBox("Cannot open themeslist.rc for writing");
		return false;
	}
	else
	{
		using namespace std;
		HANDLE hFind;
		WIN32_FIND_DATA dataFind;
		BOOL bMoreFiles = TRUE;
	
		vector<CString> l;

		//find first file in the themes directory
		hFind = FindFirstFile(utils.themesDir + "*.*", &dataFind);

		while (hFind != INVALID_HANDLE_VALUE && bMoreFiles == TRUE)
		{		
			if(dataFind.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY || ( dataFind.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY && dataFind.dwFileAttributes & FILE_ATTRIBUTE_ARCHIVE ) )
			{
				if (strcmp(dataFind.cFileName, ".") && strcmp(dataFind.cFileName, ".."))
				{
					l.push_back(dataFind.cFileName);
				}
			}
			bMoreFiles = FindNextFile(hFind, &dataFind);
		}
		FindClose(hFind);
		sort(l.begin(), l.end(), CompareStrings);
		for (vector<CString>::iterator i = l.begin(); i != l.end(); i++)
		{
			f.WriteString((CString)"*Popup \"" + *i + "\"	!execute [!lscp /switch \"" + *i + "\"]\n");
		}
		l.clear();
		f.Close();
		return true;
	}
}


//////////////////////////////////////////////////////////////////////////////////
//
// Downloading the file
//
UINT DownloadFile(LPVOID pParam)
{
	COptionInstaller *pwnd = (COptionInstaller*)pParam;
	bool bDownloaded = false;

	char filebuf[512]; // Buffer to store incoming file bytes
	CInternetSession netSession; // Our internetsession object
	CStdioFile *fTargFile = NULL; // Pointer to the target file
	CString szFile,FileSize,KBin,KBsec,szNewName, NewName,Perc; // String variables

	pwnd->GetDlgItemText(IDC_THEMEFILE,szFile); // Get the file name from editbox

	int outfs;
	try
	{
		pwnd->m_Log.AddString("Requesting file...");

		fTargFile = netSession.OpenURL(szFile,1,INTERNET_FLAG_TRANSFER_BINARY | INTERNET_FLAG_RELOAD);
		if (fTargFile == NULL)
			pwnd->m_Log.AddString("File not found.");
		else
		{
			bDownloaded = true;
			COleDateTime dlStart = COleDateTime::GetCurrentTime();
	
			int filesize = fTargFile->SeekToEnd(); // Get file size
			fTargFile->SeekToBegin(); // Move cursor back to top for reading

			outfs = filesize / 1024; // Calculate file size in kilobytes
			FileSize.Format("%d",outfs); // Format file size in kb into string
			// Create new destination file...
			szNewName = fTargFile->GetFileName(); // Get our new name!
			szNewName.Remove('?');
			if (szNewName.Find('.') < 0)
			{
				szNewName += ".lsz";
			}
			CStdioFile fDestFile;
			if (!fDestFile.Open(utils.themesDir + szNewName, CFile::modeCreate | CFile::modeWrite | CFile::typeBinary))
			{
				pwnd->m_Log.AddString("Cannot write a theme file to the disk. Operation failed");
				fTargFile->Close();
			}
			else
			{
				int byteswrite; // Number of bytes to write into the file
				int pos = 0; // Position for our progress bar
				int nperc,kbrecv; // Percentage of progress,kbs received
				double secs,kbsec; // Seconds, kb/sec

				// If filename is too long, shorten it for Window's caption status display	
				if(szNewName.GetLength() >= 10)
				{
					NewName = szNewName.Mid(0,7); // Cut it up
					NewName += "..."; // Add 3 dots
				}

				pwnd->m_Log.AddString("Downloading...");
				pwnd->m_Progress.SetRange32(0,filesize); // Set range to file size(32bit range for big files)
	
				while (byteswrite = fTargFile->Read(filebuf, 512)) // Read from file
				{
					if(pwnd->bTerminate) // See if we got a Cancel request
					{
						fDestFile.Close(); // Close our destination file
						fTargFile->Close(); // Close remote file
						delete fTargFile; // Delete the CStdioFile object to prevent leaks
						fTargFile = NULL;
						pwnd->m_Log.AddString("Download cancelled by user"); // Set satus bar text
						AfxEndThread(0); // End thread
					}
					// Calculate total seconds between start of download up until now
					COleDateTimeSpan dlElapsed = COleDateTime::GetCurrentTime() - dlStart;
					// Get seconds between starting time and current time
					secs = dlElapsed.GetTotalSeconds();
					pos = pos + byteswrite; // Set new progress position value
					fDestFile.Write(filebuf, byteswrite); // Write actual data into file
					pwnd->m_Progress.SetPos(pos); // Draw new position on progress bar
		
					// User info calculations -
					nperc = pos * 100 / filesize; // Calculate percentage
					kbrecv = pos / 1024; // Get received KB
					Perc.Format("%d",nperc); // Format percent into string
					KBin.Format("%d",kbrecv); // Format read KB into string
					if (secs > 0)
					{
						kbsec = kbrecv / secs; // Get KB download per second
						KBsec.Format("%0.1f",kbsec); // Format KB/sec into string
						pwnd->SetDlgItemText(IDC_KBSEC,KBsec + "kb/sec");
					}

					// Update GUI and all the good stuff
				
					pwnd->SetWindowText(NewName + " - " + Perc + "% | " + KBin + "kb/" + FileSize + "kb @ " + KBsec + "Kb/sec");
					pwnd->SetDlgItemText(IDC_PERC,Perc + "%");
					pwnd->SetDlgItemText(IDC_FS,"Downloaded : " + KBin + "kb/" + FileSize + "kb");
				}
				// Download ended, close file
				fDestFile.Close(); // Close our file
			}
		}
	}
	catch(CInternetException *IE) // Error did occour?
	{
		// Oh oh time to process error message and let the user know
		CString strerror; // String variable
		char error[255]; // Error message buffer
		IE->GetErrorMessage(error,255); // Get error message
		CString szError = error;
		while (szError[szError.GetLength()-1] == '\r' || szError[szError.GetLength()-1] == '\n')
		{
			szError = szError.Left(szError.GetLength()-1);
		}
		pwnd->m_Log.AddString(szError);
		pwnd->SetDlgItemText(IDC_INSTALLTHEME,"&Install theme"); // Return to normal mode
		bDownloaded = false;
		if (fTargFile)
		{
			delete fTargFile;
			fTargFile = NULL;
		}
		IE->Delete(); // Delete exception object to prevent leaks
	}
	catch(...)
	{
		pwnd->m_Log.AddString("Cannot open URL");
		pwnd->SetDlgItemText(IDC_INSTALLTHEME,"&Install theme"); // Return to normal mode
		bDownloaded = false;
		if (fTargFile)
		{
			delete fTargFile;
			fTargFile = NULL;
		}
	}
	
	pwnd->SetDlgItemText(IDC_FS,"Downloaded : ");
	pwnd->SetWindowText("File Downloader");
	pwnd->SetDlgItemText(IDC_PERC,"0%");
	pwnd->SetDlgItemText(IDC_KBSEC,"0kb/sec");

	if(bDownloaded)
	{
		pwnd->m_Log.AddString("Download complete");
		if (fTargFile)
		{
			delete fTargFile;

			if (themeinstaller.Install(utils.themesDir + szNewName, pwnd->m_Log, pwnd->IsDlgButtonChecked(IDC_DONOTSWITCH) != FALSE))
				pwnd->m_Log.AddString("Theme installed successfully");
			else
				pwnd->m_Log.AddString("Theme installation failed");
			pwnd->m_Log.AddString("");
		}
	}
	pwnd->m_Progress.SetPos(0); // Clear progress
	pwnd->m_editThemeFile.EnableWindow(TRUE);
	pwnd->SetDlgItemText(IDC_INSTALLTHEME,"Install"); // Return to normal mode
	return 1;
}
