// Utils.cpp : implementation file
//

#include "stdafx.h"
#include "utils.h"
#include "enbitmap.h"
#include "png_support.h"
#include "FontInstall.h"
#include "extractor.h"
#include "regkey.h"

#include <zlib/zlib.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// sometimes it is better to use global variables...
CUtils utils;

#define REGKEY_INIMAPPING		"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\IniFileMapping\\system.ini\\boot"
#define REGKEY_USRSHELLMAPPING	"USR:SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon"
#define REGKEY_SYSSHELLMAPPING	"SYS:Microsoft\\Windows NT\\CurrentVersion\\Winlogon"
#define REGKEY_WINLOGON			"SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\WinLogon"
#define REGKEY_EXPLORERSETTINGS	"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer"

/////////////////////////////////////////////////////////////////////////////
//
// CUtils
//

CUtils::CUtils()
{
	// LSCP dir
	toolDir = GetToolPath();

	//check if 2k
	DWORD dwWinMajor = (DWORD)(LOBYTE(LOWORD(::GetVersion())));
	bIsNT50 = ((BYTE)dwWinMajor >= 5);
}

CUtils::~CUtils()
{

}

/////////////////////////////////////////////////////////////////////
//
// Find the exe path
// return value: "x:\\path\\to\\exe\\" (with the final slash)
CString CUtils::GetToolPath()
{
	CString	rv;
	char buffer[MAX_PATH];
	GetModuleFileName(AfxGetInstanceHandle(), buffer, MAX_PATH);
	rv = buffer;
	rv = rv.Left(rv.ReverseFind('\\') +1);
	return rv;
}

////////////////////////////////////////////////////////////////////
//
// LSCP can be runned w/out LiteStep
// So we need to know some vars values
// without reading them directly
// Use the same way as Omar's LSInstaller is,
// i.e. read some Litestep dirs from the registry
// If the following keys aren't exist,
// there will be a prompt to user -
// AddUser tool will be included too :)
//
bool CUtils::DetermineDirs()
{
	const char cszLSInstallerKey[] = "Software\\Litestep\\Installer";

	HKEY hKey;
	DWORD dwError;
	DWORD strType;
	unsigned char buffer[1024];
	DWORD bufsize = 1024;
	
	dwError = RegOpenKeyEx(HKEY_CURRENT_USER, cszLSInstallerKey, 0, KEY_READ, &hKey);
	if (dwError != ERROR_SUCCESS)
	{
		return false;
	}

	strType = REG_SZ;
	bufsize = 1024;
	dwError = RegQueryValueEx(hKey,"ThemesDir",0,&strType,buffer,&bufsize);
	themesDir = buffer;
	if (themesDir.Right(1) != "\\") themesDir += "\\";

	strType = REG_SZ;
	bufsize = 1024;
	dwError = RegQueryValueEx(hKey,"PersonalDir",0,&strType,buffer,&bufsize);
	personalDir = buffer;
	if (personalDir.Right(1) != "\\") personalDir += "\\";

	strType = REG_SZ;
	bufsize = 1024;
	dwError = RegQueryValueEx(hKey,"LiteStepDir",0,&strType,buffer,&bufsize);
	litestepDir = buffer;
	if (litestepDir.Right(1) != "\\") litestepDir += "\\";
	litestepExe = litestepDir + "litestep.exe";

	RegCloseKey(hKey);
	return true;
}

//////////////////////////////////////
//
// Check if Litestep is running
//
bool CUtils::CheckLSRunning()
{
	return ::FindWindow("TApplication", "LiteStep") != NULL;
}

//////////////////////////////////////
// 
// Execute a !bang command
//
void CUtils::ExecuteBang(CString bang)
{
	HINSTANCE hinst = ::ShellExecute(NULL, "open",
		litestepExe, bang, NULL, SW_SHOWNORMAL);
}

//////////////////////////////////////
//
// Obtain the LiteStep version string
//
CString CUtils::GetLSVersion()
{
	CString	strVersion;

	bool done = false;
	VS_FIXEDFILEINFO	fixedFileInfo;

	DWORD	dwDummy;
	UINT	nSize = GetFileVersionInfoSize((LPTSTR) (LPCTSTR) litestepExe, &dwDummy);

	if (nSize)
	{
		LPBYTE pVersionInfo = new BYTE[nSize];
		if (GetFileVersionInfo((LPTSTR)(LPCTSTR) litestepExe, 0L, nSize, pVersionInfo))
		{
			LPVOID	lpVoid;
			if (VerQueryValue(pVersionInfo, "\\", &lpVoid, &nSize))
			{
				memcpy(&fixedFileInfo, lpVoid, nSize);
				done = true;
			}
		}
		delete pVersionInfo;
	}

	if (!done)
		return "";
	else
	{
		strVersion.Format("%d.%d.%d.%d",
							HIWORD (fixedFileInfo.dwFileVersionMS),
							LOWORD (fixedFileInfo.dwFileVersionMS),
							HIWORD (fixedFileInfo.dwFileVersionLS),
							LOWORD (fixedFileInfo.dwFileVersionLS));
		return strVersion;
	}
}

//////////////////////////////////////
//
// GetShellFolderPath
//
// Given a CLSID, returns the given shell folder.
// cchPath must be atleast equal to MAX_PATH.
// Does NOT quote spaces, because often times the first thing that is done with
// the returned path is to append another string to it, which doesn't work with
// quotes.
//
bool CUtils::GetShellFolderPath(int nFolder, LPTSTR ptzPath, size_t cchPath)
{
	ASSERT(cchPath >= MAX_PATH);

	IMalloc* pMalloc;
	bool bReturn = false;

    // SHGetSpecialFolderPath is not available on Win95
    // use SHGetSpecialFolderLocation and SHGetPathFromIDList instead
    if (SUCCEEDED(SHGetMalloc(&pMalloc)))
    {
        LPITEMIDLIST pidl;
        if (SUCCEEDED(SHGetSpecialFolderLocation(NULL, nFolder, &pidl)))
        {
            bReturn = SHGetPathFromIDList(pidl, ptzPath) ? true : false;            
            if (bReturn)
            {
                PathAddBackslashEx(ptzPath, cchPath);
            }
            pMalloc->Free(pidl);
        }
        pMalloc->Release();
    }
    return bReturn;
}


//////////////////////////////////////
//
// PathAddBackslashEx
//
// Checked version of PathAddBackslash which also handles quoted paths
//
// Return values:  S_OK          - backslash appended
//                 S_FALSE       - path already ended with a backslash
//                 E_OUTOFMEMORY - buffer too small
//                 E_FAIL        - other failure (invalid input string)
//
HRESULT CUtils::PathAddBackslashEx(LPTSTR ptzPath, size_t cchPath)
{
    ASSERT(cchPath > 0);
	ASSERT(cchPath <= STRSAFE_MAX_CCH);

    HRESULT hr = E_FAIL;
    size_t cchCurrentLength = 0;

    if (SUCCEEDED(StringCchLength(ptzPath, cchPath, &cchCurrentLength)))
    {
        bool bHasQuote = false;
        LPTSTR ptzEnd = ptzPath + cchCurrentLength;
        
        if ((ptzEnd > ptzPath) && (*(ptzEnd-1) == '\"'))
        {
            --ptzEnd;
            bHasQuote = true;
        }

        if (ptzEnd > ptzPath)
        {
            if (*(ptzEnd-1) == '\\')
				hr = S_FALSE;
			else
            {
                if (cchPath - cchCurrentLength <= 1)
					hr = E_OUTOFMEMORY;
				else
                {
                    if (bHasQuote) *(ptzEnd+1) = *ptzEnd;
                    *ptzEnd = '\\';
                    if (bHasQuote) ++ptzEnd;
                    ASSERT((ptzEnd - ptzPath) < cchPath);
                    *(ptzEnd+1) = '\0';
                    hr = S_OK;
                }
            }
        }
    }
    return hr;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Read a setting from the RC file. Returns empty string if failed or not found
//
CString CUtils::ReadOptionFromRC(CString rcfile, CString settingname)
{
	CStdioFile f;
	CString line;
	int pos1, pos2;
	CString lowcase_settingname = settingname;
	CString beenread;

	lowcase_settingname.MakeLower();


	if (!f.Open(rcfile,CFile::modeRead))
	{
		return "";	//access denied
	}
	while (f.ReadString(line))
	{
		utils.TrimLeft(line);
		pos1 = line.Find('\t');
		pos2 = line.Find(' ');
		if (pos1<0 && pos2<0) continue;
		if ((pos2>0 && pos2<pos1) || pos1<0) pos1 = pos2;
		beenread = line.Left(pos1);
		beenread.MakeLower();
		if (beenread == lowcase_settingname)
		{
			f.Close();
			line = line.Mid(pos1);
			utils.Trim(line);
			line.TrimLeft('\"');// doublequotes
			line.TrimRight('\"');
			return line;
		}
	}
	f.Close();
	return "";
}

//////////////////////////////////////////////////////////////////////////////////
//
// Write an option to RC: try to find it and change or add a new
// line if the setting doesn't present
//
bool CUtils::WriteOptionToRC(CString rcfile, CString settingname, CString value)
{
	CStdioFile fin, fout;
	CString line, tline;
	int pos1, pos2;
	bool bFound = false;

	CString tmpfile = rcfile+".tmp";

	CString szSpaces = "";
	for (int i=0; i<16-settingname.GetLength(); i++)
	{
		szSpaces = szSpaces + " ";
	}

	if (!fin.Open(rcfile,CFile::modeRead) ||
		!fout.Open(tmpfile,CFile::modeWrite | CFile::modeCreate))
	{
		return false;	//access denied
	}
	while (fin.ReadString(line))
	{
		if (!bFound && line != "")
		{
			tline = line;
			utils.TrimLeft(tline);
			// find the end of the first token
			pos1 = tline.Find('\t');
			pos2 = tline.Find(' ');
			if (pos1>0 || pos2>0)
			{
				if ((pos2>0 && pos2<pos1) || pos1<0) pos1 = pos2;
				if (tline.Left(pos1) == settingname)
				{
					fout.WriteString(settingname + szSpaces + value + "\n");
					bFound = true;
					continue;
				}
			}
		}
		fout.WriteString(line + "\n");
	}
	if (!bFound)
	{
		fout.WriteString(settingname + szSpaces + value + "\n");
	}
	fin.Close();
	fout.Close();
	remove(rcfile);
	rename(tmpfile,rcfile);
	return true;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Determine is theme OTS1 instead of OTS2 by presenting of theme.rc
//
bool CUtils::IsThemeOTS1Compliant(CString themename)
{
	CStdioFile f;
	const CString ots2 = themesDir + themename + "\\theme.rc";
	return !IsFileExist(ots2);
}

//////////////////////////////////////////////////////////////////////////////////
//
// Return the main theme filename: step.rc or theme.rc in according to OTS ver.
//
CString CUtils::GetFullThemeRCName(CString themename)
{
	CStdioFile f;
	const CString ots2 = themesDir + themename + "\\theme.rc";
	const CString ots1 = themesDir + themename + "\\step.rc";
	return IsFileExist(ots2) ? ots2 : ots1;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Read a setting from the theme's .ini
//
CString CUtils::ReadOptionFromINI(CString themename, CString settingname)
{
	CString themeini = themesDir + themename + "\\theme.ini";
	char buf[1024];
	GetPrivateProfileString("Install",settingname,"",buf,1024,themeini);
	return buf;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Register wallfile as wallpaper. It must be .bmp file (or .jpg if WinXP)
//
void CUtils::InstallWallpaper(CString wallfile, CString walltype)
{
	const int winfo_entries = 3;
	const struct WallpaperInfo
	{
		char stylename[10];
		char keystyle[2];
		char keytile[2];
	} winfo[] = 
	{
		{"tiled","1","1"},
		{"centered","0","0"},
		{"stretched","2","0"}
	};

	HKEY hKey;

	for (int i=0; i<winfo_entries; i++)
	{
		if (walltype == winfo[i].stylename)
		{
			if(RegOpenKeyEx(HKEY_CURRENT_USER, "Control Panel\\Desktop", 0, KEY_WRITE, &hKey) == ERROR_SUCCESS)
			{
				RegSetValueEx(hKey, "WallpaperStyle",	0, REG_SZ,
					(const unsigned char*)winfo[i].keystyle, 2);
				RegSetValueEx(hKey, "TileWallpaper",	0, REG_SZ,
					(const unsigned char*)winfo[i].keytile, 2);
				RegCloseKey(hKey);
				SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, (LPVOID)(LPCTSTR)wallfile, SPIF_UPDATEINIFILE | SPIF_SENDCHANGE);
			}
			break;
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
// Save stored in memory HBITMAP as .BMP file.
// Required by ThemeInstaller/Switcher
//
bool CUtils::SaveBitmapToFile(HBITMAP hBmp, CString pszFileName)
{
	int		 hFile;
	OFSTRUCT	ofReOpenBuff;
	HBITMAP	 hTmpBmp, hBmpOld;
	bool		bSuccess;
	BITMAPFILEHEADER	bfh;
	PBITMAPINFO pbmi;
	PBYTE	   pBits;
	BITMAPINFO  bmi;
	PBYTE pjTmp, pjTmpBmi;
	ULONG sizBMI;

	CDC dcMem;
	CDC* pDC = CWnd::GetDesktopWindow()->GetDC();
	dcMem.CreateCompatibleDC(pDC);

	bSuccess = true;
	bmi.bmiHeader.biSize = 0x28;
	bmi.bmiHeader.biBitCount = 0;

	// obtain the bitmap size
	if ((GetDIBits(dcMem,hBmp,0,0,NULL,&bmi,DIB_RGB_COLORS)) == 0)
	{
		CWnd::GetDesktopWindow()->ReleaseDC(pDC);
		return false;
	}
	// alloc enough memory to retrieve the actual bits
	if ((pBits = (PBYTE)GlobalAlloc(GMEM_FIXED | GMEM_ZEROINIT,
				bmi.bmiHeader.biSizeImage)) == NULL)
	{
		CWnd::GetDesktopWindow()->ReleaseDC(pDC);
		return false;
	}
	// 24 bits per pixel has no color table. So, we don't have to
	// allocate memory for retrieving that. Otherwise, we do.
	pbmi = &bmi;
	switch (bmi.bmiHeader.biBitCount)
	{
		case 24:
			sizBMI = sizeof(BITMAPINFOHEADER);
			break;
		case 16:
		case 32:
			sizBMI = sizeof(BITMAPINFOHEADER)+sizeof(DWORD)*3;
			break;
		default:
			sizBMI = sizeof(BITMAPINFOHEADER)+sizeof(RGBQUAD)*(1<<bmi.bmiHeader.biBitCount);
			break;
	}
	// Allocate memory for color table if it is not 24bpp...
	if (sizBMI != sizeof(BITMAPINFOHEADER))
	{
		ULONG sizTmp;
		// I need more memory for the color table
		if ((pbmi = (PBITMAPINFO)GlobalAlloc(GMEM_FIXED | GMEM_ZEROINIT, sizBMI )) == NULL)
		{
			bSuccess = false;
			goto ErrExit1;
		}
		// Let's copy the Bitmap info header data over
		pjTmp = (PBYTE)pbmi;
		pjTmpBmi = (PBYTE)&bmi;
		sizTmp = sizeof(BITMAPINFOHEADER);

		while(sizTmp--)
		{
			*(((PBYTE)pjTmp)++) = *((pjTmpBmi)++);
		}
	}
	// Let's open the file and get ready for writing
	if ((hFile = OpenFile(pszFileName, (LPOFSTRUCT)&ofReOpenBuff,
				 OF_CREATE | OF_WRITE)) == -1)
	{
		AfxMessageBox("File opening failed");
		goto ErrExit2;
	}
	// But first, fill in the info for the BitmapFileHeader
	bfh.bfType = 0x4D42;							// 'BM'
	bfh.bfSize = sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)+sizBMI+pbmi->bmiHeader.biSizeImage;
	bfh.bfReserved1 =
	bfh.bfReserved2 = 0;
	bfh.bfOffBits = sizeof(BITMAPFILEHEADER)+sizBMI;
	// Write out the file header now
	if (_lwrite(hFile, (LPSTR)&bfh, sizeof(BITMAPFILEHEADER)) == -1)
	{
		bSuccess = false;
		goto ErrExit3;
	}
	// Assume that the hDC is the DC where the bitmap would have been selected
	// if indeed it has been selected
	if (hTmpBmp = CreateCompatibleBitmap(dcMem, pbmi->bmiHeader.biWidth, pbmi->bmiHeader.biHeight))
	{
		hBmpOld = (HBITMAP)SelectObject(dcMem, hTmpBmp);
		if ((GetDIBits(dcMem, hBmp, 0, pbmi->bmiHeader.biHeight, (LPSTR)pBits, pbmi, DIB_RGB_COLORS))==0)
		{
			bSuccess = false;
			goto ErrExit4;
		}
	}
	else
	{
		AfxMessageBox("Bimap not created");
		bSuccess = false;
		goto ErrExit3;
	}
	// Now write out the BitmapInfoHeader and color table, if any
	if (_lwrite(hFile, (LPSTR)pbmi, sizBMI) == -1)
	{
		bSuccess = false;
		goto ErrExit4;
	}
	// write the bits also
	if (_lwrite(hFile, (LPSTR)pBits, pbmi->bmiHeader.biSizeImage) == -1)
	{
		bSuccess = false;
		goto ErrExit4;
	}
ErrExit4:
	SelectObject(dcMem, hBmpOld);
	DeleteObject(hTmpBmp);
ErrExit3:
	_lclose(hFile);
ErrExit2:
	GlobalFree(pbmi);
ErrExit1:
	GlobalFree(pBits);
	CWnd::GetDesktopWindow()->ReleaseDC(pDC);
	return bSuccess;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Load text file to the listbox line by line
//
void CUtils::LoadTextFile(CString szFileName, CHListBox &listbox)
{
	CStdioFile f;

	listbox.ResetContent();
	if (!f.Open(szFileName,CFile::modeRead))
	{
		listbox.AddString("Expected location: " + szFileName);
		listbox.AddString("File: " + szFileName.Mid(szFileName.ReverseFind('\\')) + " unavailable");
	}
	else
	{
		CString line;
		while (f.ReadString(line))
		{
			listbox.AddString(line);
		}
		f.Close();
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
// Write folders settings to the registry,
// i.e. litestepDir, themesDir and personalDir
// to the HKCU\SOFTWARE\LiteStep\Installer
//
void CUtils::WriteRegistry()
{
	const char cszLSInstallerKey[] = "Software\\Litestep\\Installer";

	HKEY hKey;
	DWORD dwError;	
	DWORD dwDisp;

	bool bDone = false;

	dwError = RegCreateKeyEx(HKEY_CURRENT_USER, cszLSInstallerKey, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKey, &dwDisp);

	if (dwError == ERROR_SUCCESS)
	{
		dwError = RegSetValueEx(hKey,"ThemesDir",0,REG_SZ,(CONST BYTE*)(LPCTSTR)themesDir,themesDir.GetLength()+1);
		if (dwError == ERROR_SUCCESS)
		{
			dwError = RegSetValueEx(hKey,"PersonalDir",0,REG_SZ,(CONST BYTE*)(LPCTSTR)personalDir,personalDir.GetLength()+1);
			if (dwError == ERROR_SUCCESS)
			{
				dwError = RegSetValueEx(hKey,"LiteStepDir",0,REG_SZ,(CONST BYTE*)(LPCTSTR)litestepDir,litestepDir.GetLength()+1);
				bDone = (dwError == ERROR_SUCCESS);
			}
		}
	}
	if (!bDone)
	{
		AfxMessageBox("Cannot write registry settings! :(");
	}
	RegCloseKey(hKey);
	return;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Service function, just checks if file exist
// I don't know is there a 'normal' implementation for this
//
bool CUtils::IsFileExist(CString szFileName)
{
	CStdioFile f;
	if (f.Open(szFileName,CFile::modeRead))
	{
		f.Close();
		return true;
	}
	return false;
}

//////////////////////////////////////////////////////////////////////////////////
//
// Replace $ThemesDir$ in the pattern to the full path
//
CString CUtils::ExpandString(CString pattern)
{
	CString csPattern = pattern;
	csPattern.MakeLower();
	int i;
	if ((i = csPattern.Find("$themesdir$")) < 0)
		return pattern;
	else
		return pattern.Left(i) + utils.themesDir + pattern.Mid(i + 11);
}

//////////////////////////////////////////////////////////////////////////////////
//
// Copy specified string to the clipboard
//
void CUtils::CopyToClipboard(HWND hWnd, CString csText)
{
	if(OpenClipboard(hWnd))
	{
		EmptyClipboard();
		HGLOBAL hGlobal = GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT, csText.GetLength() + 1);
		if (hGlobal)
		{
			TCHAR* lock = (LPTSTR)GlobalLock(hGlobal);
			memcpy(lock, (LPCTSTR)csText, csText.GetLength() + 1);
            GlobalUnlock(hGlobal);
			SetClipboardData(CF_TEXT, hGlobal);
		}
		CloseClipboard();
	}
}

//////////////////////////////////////////////////////////////////////////////////
//
// Delete folder and all its subfolders in the 'silent' mode
// Last character of the csFolder cannot be backslash ('\')
//
void CUtils::DeleteFolder(CString csFolder)
{
	SHFILEOPSTRUCT shfop;
	char from[MAX_PATH+1];
	StringCbCopy(from,MAX_PATH+1,csFolder.Left(csFolder.GetLength()-1));
	from[csFolder.GetLength()] = 0;
	shfop.hwnd		= NULL;
	shfop.wFunc		= FO_DELETE;
	shfop.pFrom		= from;
	shfop.pTo		= NULL;
	shfop.fFlags	= FOF_NOCONFIRMATION | FOF_SILENT;
	SHFileOperation(&shfop);
}

void CUtils::TrimLeft(CString &csString)
{
	while (csString[0] == ' ' || csString[0] == '\t')
	{
		csString.TrimLeft(' ');
		csString.TrimLeft('\t');
	}
}

void CUtils::TrimRight(CString &csString)
{
	while (csString.Right(1) == ' ' || csString.Right(1) == '\t')
	{
		csString.TrimRight(' ');
		csString.TrimRight('\t');
	}
}

void CUtils::Trim(CString &csString)
{
	TrimLeft(csString);
	TrimRight(csString);
}


//////////////////////////////////////////////////////////////////////////////////
//
// Change the current Windows shell...
//
void CUtils::SetShell(CString shellpath, bool bCurrentUserOnly)
{
	int pathlen = shellpath.GetLength();
	if (pathlen >= 5)
	{
		OSVERSIONINFO osInfo;
		ZeroMemory(&osInfo, sizeof(osInfo));
		osInfo.dwOSVersionInfoSize = sizeof(osInfo);
		GetVersionEx(&osInfo);
		if (osInfo.dwPlatformId == VER_PLATFORM_WIN32_NT)
		{
			bool bChanged = false;
			bool bNeedReboot = false;
			bool bTmpChanged;
			bool bDone = false;
			if (bCurrentUserOnly)
			{
				if (WriteKey(HKEY_LOCAL_MACHINE,REGKEY_INIMAPPING,"shell",REGKEY_USRSHELLMAPPING,bNeedReboot))
				{
					bChanged = bNeedReboot;
					if (WriteKey(HKEY_CURRENT_USER,REGKEY_WINLOGON,"shell",shellpath,bTmpChanged))
					{
						bChanged |= bTmpChanged;
						if (WriteKey(HKEY_CURRENT_USER,
									REGKEY_EXPLORERSETTINGS,
									"DesktopProcess",
									(stricmp(shellpath.Right(12),"explorer.exe") &&
									 stricmp(shellpath.Right(8), "explorer")) ? 1 : 0,
									bTmpChanged))
						{
							bChanged |= bTmpChanged;
							bDone = true;
						}
					}

				}
			} else
			{
				if (WriteKey(HKEY_LOCAL_MACHINE,REGKEY_INIMAPPING,"shell",REGKEY_SYSSHELLMAPPING,bNeedReboot))
				{
					bChanged = bNeedReboot;
					if (WriteKey(HKEY_LOCAL_MACHINE,REGKEY_WINLOGON,"shell",shellpath,bTmpChanged))
					{
						bChanged |= bTmpChanged;
						if (WriteKey(HKEY_CURRENT_USER,REGKEY_WINLOGON,"shell",shellpath,bTmpChanged))
						{
							bChanged |= bTmpChanged;
							if (WriteKey(HKEY_CURRENT_USER,
										REGKEY_EXPLORERSETTINGS,
										"DesktopProcess",
										0 == stricmp(shellpath.Right(12),"explorer.exe") ? 0 : 1,
										bTmpChanged))
							{
								bChanged |= bTmpChanged;
								bDone = true;
							}
						}
					}
				}
			}
			if (!bDone)
			{
				AfxMessageBox("Cannot write the registry settings.\nProbably security permissions conflict.\nContact your system administrator.");
			}
			else
			{
				if (bNeedReboot)
				{
					if (IDYES == AfxMessageBox("Shell written successfully, but the settings would not be applied until you restart your PC.\nDo you want to reboot it now?",MB_YESNO))
					{
						TakeShutdownPriviledges();
						ExitWindowsEx(EWX_REBOOT,0);
					}
				}
				else
				{
					int answ;
					if (bChanged)
						answ = AfxMessageBox("Shell written successfully, but the settings would not be applied until the next user session.\nDo you want to log off now?",MB_YESNO);
					else
						answ = AfxMessageBox("It is already your shell in according to the registry settings. May be you want to log off and start a new session?",MB_YESNO);
					if (answ == IDYES)
					{
						ExitWindowsEx(EWX_LOGOFF | EWX_FORCE, 0);
					}
				}
			}
		}
		else
		{
			char szWinDir[MAX_PATH];
			GetWindowsDirectory(szWinDir, sizeof(szWinDir));
			StringCchCat(szWinDir, MAX_PATH, "\\SYSTEM.INI");
			WritePrivateProfileString("Boot", "Shell", shellpath, szWinDir);
			if (IDYES == AfxMessageBox("Shell written successfully, but the settings would not be applied until you restart your PC.\nDo you want to reboot it now?",MB_YESNO))
			{
				TakeShutdownPriviledges();
				ExitWindowsEx(EWX_REBOOT | EWX_FORCE,0);
			}
		}
	}
	else
	{
		AfxMessageBox("Invalid shell file name: " + shellpath);
	}
}


bool CUtils::WriteKey(HKEY domain, CString keyname, CString valuename, CString value, bool &changed)
{
	CRegKey key;
	bool bNeed = true;
	char buf[1024];
	DWORD dwSize = 1024;

	if (key.Open(domain, keyname) != ERROR_SUCCESS)
	{
		if (key.Create(domain,keyname) != ERROR_SUCCESS) return false;	//cannot create a key
	}
	if (key.QueryValue(buf,valuename,&dwSize) == ERROR_SUCCESS)
	{
		bNeed = stricmp(buf,value) != 0;
	}
	if (bNeed)
	{
		changed = (key.SetValue(value, valuename) == ERROR_SUCCESS);
		key.Close();
		return changed;
	}
	changed = false;
	return true;
}

bool CUtils::WriteKey(HKEY domain, CString keyname, CString valuename, DWORD value, bool &changed)
{
	CRegKey key;
	bool bNeed = true;
	DWORD buf;

	if (key.Open(domain, keyname) != ERROR_SUCCESS)
	{
		if (key.Create(domain,keyname) != ERROR_SUCCESS) return false;	//cannot create a key
	}
	if (key.QueryValue(buf,valuename) == ERROR_SUCCESS)
	{
		bNeed = (buf != value);
	}
	if (bNeed)
	{
		changed = (key.SetValue(value,valuename) == ERROR_SUCCESS);
		key.Close();
		return changed;
	}
	changed = false;
	return true;
}

void CUtils::TakeShutdownPriviledges()
{
	HANDLE           hToken;
	TOKEN_PRIVILEGES tp;
	LUID             luid;
	OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken);
	LookupPrivilegeValue( NULL, SE_SHUTDOWN_NAME, &luid );
	tp.PrivilegeCount = 1;
	tp.Privileges[0].Luid = luid;
	tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
	AdjustTokenPrivileges(hToken,FALSE,&tp,sizeof(TOKEN_PRIVILEGES),NULL,NULL);
}

